import { BlockPermutation, system, world } from '@minecraft/server';

const axes = [
    'minecraft:diamond_axe',
    'minecraft:golden_axe',
    'minecraft:iron_axe',
    'minecraft:netherite_axe',
    'minecraft:stone_axe',
    'minecraft:wooden_axe'
]


function getBlockId(block) {
    const strippableBlocks = [
        { id: 'block:ghost_bamboo_block', sound: 'step.bamboo_wood' },
        { id: 'block:ghost_hyphae_crimson', sound: 'use.stem' },
        { id: 'block:ghost_hyphae_warped', sound: 'use.stem' },
        { id: 'block:ghost_log_acacia', sound: 'use.wood' },
        { id: 'block:ghost_log_birch', sound: 'use.wood' },
        { id: 'block:ghost_log_cherry', sound: 'step.cherry_wood' },
        { id: 'block:ghost_log_dark_oak', sound: 'use.wood' },
        { id: 'block:ghost_log_jungle', sound: 'use.wood' },
        { id: 'block:ghost_log_mangrove', sound: 'use.wood' },
        { id: 'block:ghost_log_oak', sound: 'use.wood' },
        { id: 'block:ghost_log_spruce', sound: 'use.wood' },
        { id: 'block:ghost_stem_crimson', sound: 'use.stem' },
        { id: 'block:ghost_stem_warped', sound: 'use.stem' },
        { id: 'block:ghost_wood_acacia', sound: 'use.wood' },
        { id: 'block:ghost_wood_birch', sound: 'use.wood' },
        { id: 'block:ghost_wood_cherry', sound: 'step.cherry_wood' },
        { id: 'block:ghost_wood_dark_oak', sound: 'use.wood' },
        { id: 'block:ghost_wood_jungle', sound: 'use.wood' },
        { id: 'block:ghost_wood_mangrove', sound: 'use.wood' },
        { id: 'block:ghost_wood_oak', sound: 'use.wood' },
        { id: 'block:ghost_wood_spruce', sound: 'use.wood' },
    ]

    for(const blockData of strippableBlocks) {
        if(block.permutation.matches(blockData.id)) {
            return blockData;
        }
    }
}

world.beforeEvents.itemUseOn.subscribe(data => {
    const {itemStack, block, source: player} = data;

    if(axes.includes(itemStack.typeId)) {
        const blockData = getBlockId(block);
        if(blockData) {
            system.run(() => {
                const strippedBlockId = blockData.id.replace('block:ghost_', 'block:ghost_stripped_');
                block.setPermutation(BlockPermutation.resolve(strippedBlockId, block.permutation.getAllStates()));
                player.playSound(blockData.sound, { pitch: 1, location: block.location, volume: 1 });
                looseDurability(itemStack);
            });
        }
    }
    else if(itemStack.typeId === 'minecraft:flint_and_steel') {
        if(block.permutation.matches('block:ghost_tnt')) {
            data.cancel = true;
            system.run(() => {
                block.setPermutation(BlockPermutation.resolve('minecraft:air'));
                block.dimension.spawnEntity('minecraft:tnt', block.location);
                looseDurability(itemStack);
            });
        }
    }

    function looseDurability(item) {
        const equippable = player.getComponent('equippable');
        const durability = item.getComponent('durability');

        if(player.matches({gameMode: 'creative'})) return;
        if(durability.maxDurability < durability.damage) {
            durability.damage = 1;
            equippable.setEquipment('Mainhand', item);
        }
        else if((durability.maxDurability - durability.damage) <= 1) {
            equippable.setEquipment('Mainhand', undefined);
        }
        else {
            durability.damage++
            equippable.setEquipment('Mainhand', item);
        }
    }
});

world.beforeEvents.explosion.subscribe(data => {
    const blocks = data.getImpactedBlocks();
    blocks.forEach(block => {
        if(block.permutation.matches('block:ghost_tnt')) {
            system.run(() => {
                data.dimension.spawnEntity('minecraft:tnt', block.location);
            });
        }
    });
});