import { world } from "@minecraft/server";


// Detectar el bloque más cercano del mismo tipo en la dirección y+
export function detectUpBlocks(dimension, startLocation, blockTypes) {
  let y = startLocation.y + 1; // Comenzar justo arriba de la posición inicial
  const maxY = startLocation.y + 64; // Altura máxima que alcanza el elevador

  while (y <= maxY) {
    const currentLocation = { x: startLocation.x, y: y, z: startLocation.z };
    const currentBlock = dimension.getBlock(currentLocation);

    if (blockTypes.includes(currentBlock.typeId)) {
      if (!currentBlock.above().isSolid && !currentBlock.above().above().isSolid) {
          // Devuelve la ubicación justo encima del bloque encontrado
          return { x: currentLocation.x, y: y + 1, z: currentLocation.z };
      }
    }
    y++;
  }

  return null;
}

export function detectDownBlocks(dimension, startLocation, blockTypes) {
  let y = startLocation.y - 1; // Comenzar justo abajo de la posición inicial
   // Altura minima que alcanza el elevador
  const minY = Math.max(y - 64, -64);

  while (y >= minY) {
      const currentLocation = { x: startLocation.x, y: y, z: startLocation.z };
      const currentBlock = dimension.getBlock(currentLocation);

      if (blockTypes.includes(currentBlock.typeId)) {
        if (!currentBlock.above().isSolid && !currentBlock.above().above().isSolid) {
          // Devuelve la ubicación justo debajo del bloque encontrado
          return { x: currentLocation.x, y: y + 1, z: currentLocation.z };
      }
      }

      y--;
  }

  return null;
}

export function teleportPlayer(player, location, facingDirection) {
  const roundedCoords = {
    x: location.x,
    y: Math.floor(location.y),
    z: location.z
  }
  // + 0.0640
  if (player && roundedCoords) {
    const dimension = player.dimension;
    //const soundCommand = `playsound block.teleport @a[r=8] ${roundedCoords.x} ${roundedCoords.y} ${roundedCoords.z} 1 1 5`;

    player.playSound("block.teleport", { location: player.location });

    let enableDirection = true
    if (facingDirection == false){
      enableDirection = false
    }

    if(enableDirection){
      player.teleport(roundedCoords, {
        dimension: player.dimension,
        keepVelocity: false,
        rotation: facingDirection
      });
    }
    else{
      player.teleport(roundedCoords, {
        dimension: player.dimension,
        keepVelocity: false
      });
    }

  }
}

export function getDirection(blockLocation, player){
  const location = { x: blockLocation.x, y: blockLocation.y - 1, z: blockLocation.z }
  const block = player.dimension.getBlock(location)
  const blockDirection = block.permutation.getState('op:directional')
  let facingDirection;
  if (blockDirection == "none"){
    facingDirection = false
  }
  else if (blockDirection == "north"){
    facingDirection = { x: 0, y: 180, z: 0}
  }
  else if(blockDirection == "south"){
    facingDirection = { x: 0, y: 0, z: 0}
  }
  else if(blockDirection == "west"){
    facingDirection = { x: 0, y: 90, z: 0}
  }
  else if(blockDirection == "east"){
    facingDirection = { x: 0, y: -90, z: 0}
  }
  return facingDirection
}

const playerSneakingMap = new Map();
const elevatorBlocks = [
  "elevatorid:elevator_white",
  "elevatorid:elevator_black",
  "elevatorid:elevator_blue",
  "elevatorid:elevator_brown",
  "elevatorid:elevator_cyan",
  "elevatorid:elevator_gray",
  "elevatorid:elevator_green",
  "elevatorid:elevator_light_blue",
  "elevatorid:elevator_light_gray",
  "elevatorid:elevator_lime",
  "elevatorid:elevator_magenta",
  "elevatorid:elevator_orange",
  "elevatorid:elevator_pink",
  "elevatorid:elevator_purple",
  "elevatorid:elevator_red",
  "elevatorid:elevator_yellow"
];

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockTypeRegistry.registerCustomComponent('elevatorid:teleport', {
        onTick: e => {
          const allPlayers = world.getAllPlayers();
          for (const player of allPlayers) {
            const playerLocation = player.location;
            const block = e.block;
            const blockBelowLocation = {
              x: playerLocation.x,
              y: playerLocation.y - 1,
              z: playerLocation.z
            };
            const blockBelowPlayer = player.dimension.getBlock(blockBelowLocation)
            if ( block.typeId === blockBelowPlayer.typeId) {
              const playerId = player.id;
              if (player.isSneaking){
                if (!playerSneakingMap.get(playerId)){
          
                  const closestBlockBelow = detectDownBlocks(player.dimension, blockBelowLocation, elevatorBlocks);
                  if(!closestBlockBelow){
                    return
                  }
                  const facingDirection = getDirection(closestBlockBelow, player)
                  if (closestBlockBelow){
                    teleportPlayer(player, closestBlockBelow, facingDirection);
                  }
                  playerSneakingMap.set(playerId, true);
                }
              }
              else {
                playerSneakingMap.set(playerId, false);
              }
            }
          }
        },
        onStepOff: e => {
          const { entity, block, dimension } = e;
          if (entity.typeId === 'minecraft:player') {
            if (block.permutation.getState("op:toggletick") === true) {
              block.setPermutation(block.permutation.withState("op:toggletick",false));
            }
            const blockBelowLocation = {
              x: entity.location.x,
              y: block.location.y,
              z: entity.location.z
            };
            if (entity.isJumping){            
              const closestBlockAbove = detectUpBlocks(entity.dimension, blockBelowLocation, elevatorBlocks);
              if(!closestBlockAbove){
                 return
              }
              const facingDirection = getDirection(closestBlockAbove, entity)
              if (closestBlockAbove){
                teleportPlayer(entity, closestBlockAbove, facingDirection);
              }
            } 
          }
        },
        onStepOn: e => {
          const { entity, block, dimension } = e;
          if (entity.typeId === 'minecraft:player'){
            if (block.permutation.getState("op:toggletick") === false) {
              block.setPermutation(block.permutation.withState("op:toggletick",true));
            }
          }
        },

    })
})