import { world, EquipmentSlot, GameMode } from "@minecraft/server";
import { EntityUtils } from "./entityUtils";
import { ItemStackUtils } from "./itemStackUtils";

export class PlayerUtils extends EntityUtils {
    static #getEquipCopm(player) {
        return player.getComponent("minecraft:equippable");
    }

    static isCreativeMode(player) {
        const target = Array.from(world.getPlayers({
            name: player.name,
            gameMode: GameMode.creative
        }));
        return target.length > 0 ? true : false;
    }

    static getMainhandSlot(player) {
        return this.#getEquipCopm(player).getEquipmentSlot(EquipmentSlot.Mainhand);
    }

    static getMainhandItem(player) {
        return this.#getEquipCopm(player).getEquipment(EquipmentSlot.Mainhand);
    }

    static setMainhandItem(player, itemStack, ignoreGameMode = false) {
        if (ignoreGameMode || !ignoreGameMode && !this.isCreativeMode(player)) {
            this.getMainhandSlot(player).setItem(itemStack);
        };
    }

    static damageMainhandItem(player, damage = 1, ignoreGameMode = false, ignoreUnbreakingEnchantment = false) {
        if (ignoreGameMode || !ignoreGameMode && !this.isCreativeMode(player)) {
            const slot = this.getMainhandSlot(player);
            const damageSuccess = ItemStackUtils.damageDurability(slot, damage, ignoreUnbreakingEnchantment);
            if(damageSuccess && !slot.getItem()) {
                //player.dimension.playSound("random.break", player.location);
                const loc = player.location;
                player.dimension.runCommand(`playsound random.break @a ${loc.x + 0.5} ${loc.y + 0.5} ${loc.z + 0.5}`);
            };
        };
    }

    static decrementSelectedStack(player, ignoreGameMode = false) {
        if (ignoreGameMode || !ignoreGameMode && !this.isCreativeMode(player)) {
            ItemStackUtils.decrementStack(this.getMainhandSlot(player));
        };
    }
}