import { Direction } from "@minecraft/server";
import { Axis } from "../enum/system";
import { VECTOR3_UP, VECTOR3_DOWN, VECTOR3_NORTH, VECTOR3_SOUTH, VECTOR3_WEST, VECTOR3_EAST } from "../enum/vector";
import { Vector3Utils } from "./vector3Utils";
import { Vector3Builder } from "../build/vector3Builder";

export function capitalize(str) {
    return str.replace(/( |^)[a-z]/g, (L) => L.toUpperCase());
}

export function lowerCaseFirst(str) {
    return str.replace(/( |^)[A-Z]/g, (U) => U.toLowerCase());
}

// @minecraft/math.clampNumber
// https://www.npmjs.com/package/@minecraft/math
// Licensed under the MIT License.

/**
 * Clamps the passed in number to the passed in min and max values.
 *
 * @public
 */
export function clampNumber(val, min, max) {
    return Math.min(Math.max(val, min), max);
}

export class Utils {    
    static axis(dir) {
        const value = capitalize(dir);
        if (value == Direction.East || value == Direction.West) {
            return Axis.X;
        } else if (value == Direction.Down || value == Direction.Up) {
            return Axis.Y;
        } else if (value == Direction.North || value == Direction.South) {
            return Axis.Z;
        };
    }

    static aimLoc(vec3, dir) {
        switch (capitalize(dir)) {
            case Direction.Up: return Vector3Utils.add(vec3, VECTOR3_UP);
            case Direction.Down: return Vector3Utils.add(vec3, VECTOR3_DOWN);
            case Direction.North: return Vector3Utils.add(vec3, VECTOR3_NORTH);
            case Direction.South: return Vector3Utils.add(vec3, VECTOR3_SOUTH);
            case Direction.West: return Vector3Utils.add(vec3, VECTOR3_WEST);
            case Direction.East: return Vector3Utils.add(vec3, VECTOR3_EAST);
            default: return null;
        };
    }

    static singleFaceLocation(dir, faceLoc) {
        const surface = {...faceLoc};
        delete surface[this.axis(dir)];
        const loc = {
            x: Object.values(surface)[0],
            y: Object.values(surface)[1]
        };
        return loc;
    }

    static transToWorldLoc(vec3) {
        let integerNum = 0;
        for(const axis in vec3) {
            if(Number.isInteger(vec3[axis])) {
                integerNum++;
            } else {
                return null;
            };
        };
        if(integerNum == 3) {
            return Vector3Utils.add(vec3, new Vector3Builder(0.5, 0.5, 0.5));
        };
        return null;
    }
}