import { Direction } from "@minecraft/server";
import { Axis } from "../../library/enum/system";
import { COPPER_CAN_INTERACT } from "../../library/enum/item";
import { CopperTransformationType } from "../../library/enum/system";
import { capitalize, Utils } from "../../library/utils/systemUtils";
import { BlockUtils } from "../../library/utils/blockUtils";
import { VerticalSlab } from "./verticalSlab";
import { PlayerUtils } from "../../library/utils/playerUtils";

const CARDINAL_DIRECTION_IN_ORDER = [
    Direction.North,
    Direction.East,
    Direction.South,
    Direction.West
];

export class VerticalSlabSet extends VerticalSlab {
    #playerAxis

    constructor(onSet) {
        super(onSet.block);
        this.selectedBlock = onSet.block;
        this.blockFace = onSet.blockFace;
        this.aimVerticalSlab = new VerticalSlab(BlockUtils.getAimBlock(this.dimension, this.location, this.blockFace));
        this.faceLocation = onSet.faceLocation;
        this.source = onSet.source;
        this.#playerAxis = PlayerUtils.getEntityAxis(this.source);
        this.itemStack = onSet.itemStack;
        this.itemId = this.itemStack.typeId;
    }

    inetractCopper() {
        const itemIndex = COPPER_CAN_INTERACT.indexOf(this.itemId);
        if (itemIndex == 0) {
            this.transCopper(CopperTransformationType.WAX);
        } else if (itemIndex >= 1) {
            this.transCopper(CopperTransformationType.SCRAPE);
        };
    }

    evaluateDirection() {
        const faceLoc = Utils.singleFaceLocation(this.blockFace, this.faceLocation);
        const blockAxis = Utils.axis(this.blockFace);
        if (this.#playerAxis == Axis.Z) {
            if (faceLoc.y < 0.5 && blockAxis != Axis.Z || this.blockFace == Direction.South) {
                return Direction.North;
            } else if (faceLoc.y >= 0.5 && blockAxis != Axis.Z || this.blockFace == Direction.North) {
                return Direction.South;
            };
        } else if (this.#playerAxis == Axis.X) {
            if (faceLoc.x < 0.5 && blockAxis != Axis.X || this.blockFace == Direction.East) {
                return Direction.West;
            } else if (faceLoc.x >= 0.5 && blockAxis != Axis.X || this.blockFace == Direction.West) {
                return Direction.East;
            };
        } else return Direction.North;
    }

    evaluateMergeCondition() {
        if (VerticalSlab.checkIsVerticalSlab(this.itemId)) {
            let target, canMerge;
            const selectedBlockId = this.id;
            const aimBlockId = this.aimVerticalSlab.id;
            if (selectedBlockId == this.itemId) {
                const isAimVerticalSlab = aimBlockId == this.itemId ? true : false;
                const verticalSlabDir = capitalize(this.getDirection());
                const isAnotherSide = Math.abs(CARDINAL_DIRECTION_IN_ORDER.indexOf(verticalSlabDir) - CARDINAL_DIRECTION_IN_ORDER.indexOf(this.blockFace)) == 2;
                if (this.blockFace != Direction.Down && this.blockFace != Direction.Up && isAnotherSide) {
                    canMerge = true;
                    target = this;
                } else {
                    canMerge = isAimVerticalSlab;
                    target = canMerge ? this.aimVerticalSlab : undefined;
                };
            } else if (selectedBlockId != this.itemId && aimBlockId == this.itemId) {
                canMerge = true;
                target = this.aimVerticalSlab;
            } else {
                canMerge = false;
            };
            if (canMerge) {
                return target;
            };
        };
    }

    evaluateWaterlogCondition() {
        if (this.itemId == "minecraft:water_bucket") {
            if (VerticalSlab.checkIsVerticalSlab(this.id)) {
                return this;
            }/* else if (VerticalSlab.checkIsVerticalSlab(this.aimVerticalSlab.id) && !this.selectedBlock.type.canBeWaterlogged) {
                return this.aimVerticalSlab;
            }*/;
        };
    }
}
