import { system, world } from "@minecraft/server";
import { Utils } from "../../../library/utils/systemUtils";
import { VerticalSlab } from "../../block/verticalSlab";
import { VerticalSlabSet } from "../../block/verticalSlabSet";
import { PlayerUtils } from "../../../library/utils/playerUtils";


export class VerticalSlabMergeEvent {
    #delay = false;
    #canInteract = false;
    #canPlace = false;

    constructor() {
        this.onPlacing;
        this.item;
        this.source;
        this.targetToMerge;
    }

    setup() {
        world.beforeEvents.itemUseOn.subscribe(data => {
            this.onPlacing = new VerticalSlabSet(data);
            this.source = data.source;
            this.item = this.onPlacing.itemId;
            this.#beforeUseOn(data);
        });
        world.afterEvents.itemStartUseOn.subscribe(() => this.#startUseOn());
        world.afterEvents.itemUse.subscribe(() => this.#afterUse());
        world.afterEvents.itemStopUseOn.subscribe(() => this.#stopUseOn());
        world.afterEvents.playerPlaceBlock.subscribe(() => this.#placeBlock());
    }

    #beforeUseOn(data) {
        if (this.#delay) {
            data.cancel = true;
        };
        this.isVerticalSlab = false;
        this.#canPlace = false;
        this.targetToMerge = undefined;
        if (VerticalSlab.checkIsVerticalSlab(this.item)) {
            this.isVerticalSlab = true;
            this.targetToMerge = this.onPlacing.evaluateMergeCondition();
            let isSelectingVerticalSlab = false;
            if (JSON.stringify(this.targetToMerge?.location) == JSON.stringify(this.onPlacing.location)) {
                isSelectingVerticalSlab = true;
                data.cancel = true;
            };
            system.runTimeout(() => {
                if ((isSelectingVerticalSlab ? true : !this.#canInteract) && this.targetToMerge != undefined) {
                    const mergeSuccess = this.targetToMerge.merge();
                    if(mergeSuccess) {
                        world.playSound(this.targetToMerge.data.sound, Utils.transToWorldLoc(this.targetToMerge.location));
                        PlayerUtils.decrementSelectedStack(this.source);
                    };
                    this.delay = true;
                    system.runTimeout(() => this.delay = false, 3);
                }
            }, 1)
        }
    }

    #startUseOn() {
        this.#canInteract = true;
    }

    #afterUse() {
        if (!this.#delay && !this.#canPlace && this.isVerticalSlab && this.targetToMerge != undefined) {
            const mergeSuccess = this.targetToMerge.merge();
            if(mergeSuccess) {
                world.playSound(this.targetToMerge.data.sound, Utils.transToWorldLoc(this.targetToMerge.location));
                PlayerUtils.decrementSelectedStack(this.source);
            };
            this.#delay = true;
            system.runTimeout(() => {
                this.#delay = false;
            }, 3);
        }
    }

    #stopUseOn() {
        system.runTimeout(() => {
            this.#canInteract = false;
        }, 1);
    }

    #placeBlock() {
        if (this.isVerticalSlab) {
            system.runTimeout(() => {
                this.#canInteract = false;
            }, 1);
            this.#canPlace = true;
        }
    }
}

export const verticalSlabMerge = new VerticalSlabMergeEvent();