import { world } from "@minecraft/server";
import { VERTICAL_SLAB_STATES } from "../../../library/data/stateData";
import { lowerCaseFirst } from "../../../library/utils/systemUtils";
import { BlockUtils } from "../../../library/utils/blockUtils";
import { VerticalSlab } from "../../block/verticalSlab";
import { VerticalSlabSet } from "../../block/verticalSlabSet";

export class VerticalSlabPlaceEvent {
    #hasWater = new Map();

    constructor() {
        this.onPlacing;
        this.verticalSlabDir = "north";
        this.item;
    }

    setup() {
        world.beforeEvents.itemUseOn.subscribe(data => {
            this.onPlacing = new VerticalSlabSet(data);
            this.item = this.onPlacing.itemId;
            this.#beforeUseOn(data);
        });
        world.afterEvents.playerPlaceBlock.subscribe(data => this.#placeBlock(data));
    }

    #beforeUseOn(data) {
        this.isVerticalSlab = false;
        const selectedBlock = data.block;
        const aimBlock = BlockUtils.getAimBlock(this.onPlacing.dimension, this.onPlacing.location, this.onPlacing.blockFace);
        if (VerticalSlab.checkIsVerticalSlab(this.item)) {
            this.isVerticalSlab = true;

            /*
            * Whether the vertical slab is waterlogged after placing it. 
            */
            const isStillWater = aimBlock.permutation.matches("minecraft:water") && aimBlock.permutation.getState("liquid_depth") == 0;
            this.#hasWater.set(JSON.stringify(selectedBlock.location), selectedBlock.isWaterlogged);
            this.#hasWater.set(JSON.stringify(aimBlock.location), isStillWater ? true : aimBlock.isWaterlogged);

            /*
            * Place the vertical slab and set the states.
            */
            this.verticalSlabDir = lowerCaseFirst(this.onPlacing.evaluateDirection());
        }
    }

    #placeBlock(data) {
        if (this.isVerticalSlab) {
            const block = new VerticalSlab(data.block);
            if (this.#hasWater.get(JSON.stringify(block.location))) {
                block.waterlog();
            };
            block.setStates(BlockUtils.createStateRecord(VERTICAL_SLAB_STATES, this.verticalSlabDir, false));
        }
    }
}

export const verticalSlabPlace = new VerticalSlabPlaceEvent();