import { system, world, ItemStack } from "@minecraft/server";
import { Utils } from "../../../library/utils/systemUtils";
import { PlayerUtils } from "../../../library/utils/playerUtils";
import { VerticalSlabSet } from "../../block/verticalSlabSet";

export class VerticalSlabWaterlogEvent {
    #canWaterlog = true;
    success = false;

    constructor() {
        this.item;
        this.waterlogData;
        this.source;
    }
    
    setup() {
        world.beforeEvents.itemUseOn.subscribe(data => {
            this.waterlogData = new VerticalSlabSet(data);
            this.source = data.source;
            this.item = this.waterlogData.itemId;
            this.#waterlog(data);
        });
        world.afterEvents.playerPlaceBlock.subscribe(() => this.#onPlayerPlaceBlock());
    }

    #waterlog(data) {
        this.#canWaterlog = true;
        const targetToWaterlog = this.waterlogData.evaluateWaterlogCondition();
        if (targetToWaterlog != undefined) {
            const waterlogLoc = targetToWaterlog.location;
            const aimLoc = this.waterlogData.aimBlock.location;
            this.success = false;
            if(JSON.stringify(waterlogLoc) == JSON.stringify(aimLoc) && this.#canWaterlog) {
                system.run(() => {
                    this.success = targetToWaterlog.waterlog();
                });
            } else if(JSON.stringify(waterlogLoc) == JSON.stringify(this.waterlogData.location)) {
                data.cancel = true;
                system.run(() => {
                    this.success = targetToWaterlog.waterlog();
                });
            };
            system.run(() => {
                if(this.success) {
                    world.playSound("bucket.empty_water", Utils.transToWorldLoc(targetToWaterlog.location));
                    PlayerUtils.setMainhandItem(this.source, new ItemStack("minecraft:bucket", 1));
                };
            });
        }
    }

    #onPlayerPlaceBlock() {
        if (this.item == "minecraft:water_bucket") {
            this.#canWaterlog = false;
        };
    }
}

export const verticalSlabWaterlog = new VerticalSlabWaterlogEvent();